<?php 
namespace Concrete\Package\CcOptinFree\Contexts;

use Page;
use Loader;
use URL;
use Concrete\Core\Asset\Asset;
use Concrete\Core\Asset\AssetList;
use Concrete\Core\Http\ResponseAssetGroup;

use Concrete\Package\CcOptinFree\CcOptin;
use Concrete\Package\CcOptinFree\CcOptinCampaignTracker;
use Concrete\Package\CcOptinFree\CcOptinActiveCampaigns;

use Concrete\Package\CcOptinFree\Models\CcOptinCampaign;
use Concrete\Package\CcOptinFree\Models\CcOptinType;
use Concrete\Package\CcOptinFree\Models\CcOptinDesign;

class DisplayCampaigns
{
	private $page;
	private $campaignIdList;
	private $userAgent;
	private $assets;

	public function __construct($page) {
		$this->page = $page;
		$this->userAgent = $_SERVER['HTTP_USER_AGENT'];
		$this->assets = ResponseAssetGroup::get();

		$activeCampaigns = new CcOptinActiveCampaigns($page);
		$this->campaignIdList = $activeCampaigns->get();
	}

	public function count() {
		return count($this->campaignIdList);
	}

	public function display() {
		$displayCampaigns = false;
		$assets = array();

		foreach ($this->campaignIdList as $id) {
			$campaign = new CcOptinCampaign();
			$campaign->load('id = ?', array($id));

			$currentCampaign = new DisplayCampaign($campaign, $this->page);
			
			if($currentCampaign->display()) {
				$displayCampaigns = true;
				$assets = array_merge($assets, $currentCampaign->assets());
			}
		}

		if($displayCampaigns) {
			// Add the basic css and js dependencies
			$this->assets_require('css', 'cc-template-reset');
			$this->assets_require('javascript', 'cc-display-campaign');

			if($this->userAgent_isLowIE()) {
				$this->assets_require('javascript', 'cc-placeholder');
			}

			foreach ($assets as $asset) {
				$this->assets_require($asset);
			}
		}

		return $displayCampaigns;
	}

	private function assets_require($type, $handle = null) {
		// Fix for mixed asset/assetpointer sorting not working according to
		// https://github.com/concrete5/concrete5-5.7.0/issues/1433
		if(is_array($type)) {
			$assets = AssetList::getInstance();
			$assets->register($type[0], $type[1], $type[1], array(), CcOptin::package());
			return $this->assets->requireAsset($type[0], $type[1]);
		}

		if($handle == null) {
			$handle = $type;
			$type = 'javascript';
		}

		$this->assets->requireAsset($type, $handle);
	}

	private function userAgent_isLowIE() {
		return strpos($this->userAgent, 'MSIE 9.') !== false || strpos($this->userAgent, 'MSIE 8.') !== false;
	}
}

class DisplayCampaign
{
	private $campaign;
	private $page;
	private $type;	
	private $design;
	private $tracker;
	private $submitActions;
	private $assets;

	public function __construct($campaign, $page) {
		$this->campaign = $campaign;
		$this->page = $page;
		$this->assets = ResponseAssetGroup::get();
		$this->tracker = new CcOptinCampaignTracker($campaign);
		$this->submitActions = $this->campaign->submitActionsArray(false);

		// Optimizing some DB access
		if(!$this->tracker->subscribed()) {
			$this->type = CcOptinType::create($this->campaign->typeClass, $this->campaign->typeID);
			$this->design = CcOptinDesign::create($this->campaign->designClass, $this->campaign->designID);
		}
	}

	public function assets() {
		$output = array(array('css', $this->design->localPath('view.css')));
		if(count($this->submitActions)) $output[] = 'cc-ajaxform';

		return array_merge($output, $this->type->dependencies());
	}

	public function display() {
		if(!$this->campaign_shouldBeDisplayed()) return false;
		$this->design_addContent();
		return true;
	}

	///// Design role /////

	private function design_addContent() {
		$uh = Loader::helper('concrete/urls');
		$json = Loader::helper('json');

		$cssId = 'cc-optin-campaign-' . $this->campaign->id;

		// The params should match html attributes in the view.
		// Strip the "CcOptin" part of the type class to convert it to lowercase underscore format.
		$typeRef = new \ReflectionClass($this->type);
		$type = $typeRef->getShortName();

		$params = array(
			'id' => $cssId,
			'class' => 'cc-optin-reset cc-optin-' . strtolower(substr($type, 7)) . ' cc-optin-' . $this->design->templateHandle,
			'action' => URL::to('/cc_optin/subscribe') . 
				'?cID=' . $this->page->getCollectionID() .
				'&id=' . $this->campaign->id
		);

		// design->content() must be a class method since it uses $this within the views.
		$this->assets->addFooterAsset($this->design->content($params));

		$trackerId = $this->tracker->id();
		$data = $this->type->jsonData();
		$rules = $json->encode($this->campaign->rulesArray());
		$submitActions = $json->encode($this->submitActions);

		$js = "<script type='text/javascript'>$(function() { " . 
			  "window.ccOptin.displayCampaign = new ccOptin.DisplayCampaign('$trackerId', '#$cssId', '$type', $data, $rules, $submitActions); " .
			  "window.ccOptin.displayCampaign.display(); " .
			  "});</script>";

		$this->assets->addFooterAsset($js);
	}

	///// Campaign role /////

	private function campaign_shouldBeDisplayed() {
		// Rules and campaign date are checked already in database when using CcOptinActiveCampaigns.
		return !$this->tracker->subscribed() && !$this->design->isEmpty();
	}
}