<?php 
namespace Concrete\Package\CcOptinFree\Models;

use Concrete\Core\Legacy\DatabaseItemList;
use Concrete\Core\Search\Column\Set as DatabaseItemListColumnSet;

abstract class CcOptinDatabaseItemList extends DatabaseItemList
{
	/**
	* The SELECT/FROM/JOIN part of the query.
	*/
	protected $query;

	protected $_columns;
	protected $_getCalled = false;
	protected $queryStringPagingVariable = 'page';

	public function __construct($columns, $defaultSortColumn = null) {
		if(empty($columns)) return;

		$this->_columns = new DatabaseItemListColumnSet();
		$this->autoSortColumns = array();

		foreach ($columns as $column) {
			if($column->isColumnSortable())
				$this->autoSortColumns[] = $column->getColumnKey();

			$this->_columns->addColumn($column);
		}

		if($defaultSortColumn) {
			$col = $this->_columns->getColumnBykey($defaultSortColumn);

			$this->_columns->setDefaultSortColumn($col);
			$this->sortBy($col->getColumnKey(), $col->getColumnDefaultSortDirection());
		}
	}

	/**
	* A subclass should override this method if className and data are needed for get() and getPage().
	*/
	public function get($itemsToGet = 0, $offset = 0, $className = null, $data = null) {
		$this->_getCalled = true;

		$output = parent::get($itemsToGet, $offset);		
		if($className === null) return $output;

		$objects = array();
		foreach ($output as $item) {
			if($data === null)
		 		$objects[] = new $className($item);
		 	else
		 		$objects[] = new $className($item, $data);
		}
		return $objects;		
	}

	public function columns() {	
		if(!$this->_getCalled) throw new \Exception("get() or getPage() must be called before columns() to enable proper sorting.");
		return $this->_columns->getColumns(); 
	}
}
